#include "adfX355.h"

ADFX355::ADFX355(uint32_t tcxo, uint64_t VCOMIN, uint64_t VCOMAX, uint32_t pfdTarget, uint8_t type) {
    this->tcxo = tcxo;
    this->type = type;
    this->VCOMIN = VCOMIN;
    this->VCOMAX = VCOMAX;

    /*
     * PFD Calculation to be validated
     *
     * Fpfd = refin*(1+D)/(R*(1+T))
     * R = refin*(1+D)/(Fpfd*(1+T))
     *
     */
    if(tcxo >= pfdTarget * 2) {
        this->mul2 = 0;
        this->div2 = 1;
    } else if(tcxo >= pfdTarget && tcxo < pfdTarget * 2) {
        this->mul2 = 0;
        this->div2 = 0;
    } else if(tcxo <= pfdTarget/2) {
        this->mul2 = 1;
        this->div2 = 0;
    } else if(tcxo < pfdTarget && tcxo > pfdTarget/2) {
        this->mul2 = 0;
        this->div2 = 0;
    }

    uint32_t tmpR = (double)tcxo * ((1+(double)this->mul2)/((double)pfdTarget*(1+(double)this->div2)));
    if(tmpR > 1023) {
        this->R = 1023;
    } else if(tmpR < 1) {
        this->R = 1;
    } else {
        this->R = tmpR;
    }

    this->PFD = ((double)tcxo * (double)((1+(double)this->mul2)/((double)this->R*(1+(double)this->div2))));
}

void ADFX355::initFreqRange(uint64_t startFreq, uint64_t stopFreq, uint64_t nbPoints){
    this->nbPoints = nbPoints;
	
	uint64_t freq = 0;

    uint64_t step = (stopFreq - startFreq) / (nbPoints - 1);
    this->freqSettings = new ADFParam[nbPoints];

    for(uint64_t i = 0; i < nbPoints; i++) {
        if(i == nbPoints-1)
            freq = stopFreq;
        else
            freq = startFreq + i * step;
		
		this->freqSettings[i].freq = freq;
		this->computeParams(this->freqSettings[i]);
    }
}

uint64_t ADFX355::getCalcFreq(ADFParam * settings) {
    return (((double)settings->INT + ((double)settings->FRAC1+((double)settings->FRAC2/(double)settings->MOD2))/(double)FixedMOD1)*(double)settings->PFD)/(double)(1 << settings->divOut);
}

ADFError ADFX355::computeParams(ADFParam * settings) {
	uint64_t vcoFreq = 0;
	settings->outx2 = 0;
	
	if((settings->freq <= VCOMAX * 2) && (settings->freq > VCOMAX)) {
		settings->outx2 = 1;
		settings->freq = settings->freq / 2;
	} else if(settings->freq > VCOMAX * 2) {
		return ADF_FREQERROR;
	}
	
	for(uint8_t i = 0; i < 7; i++) {
		if((freq * (1 << i)) >= VCOMIN && (freq * (1 << i)) <= VCOMAX) {
			vcoFreq = settings->freq * (1 << i);
			settings->divOut = i;
			break;
		}
	}
	
	double N = (double)vcoFreq / (double)settings->PFD;
	settings->INT = (uint64_t)N;
	double FRAC = N - (double)settings->INT;
	
}