#ifndef ADFX355_H
#define ADFX355_H

#include <stdint.h>

const uint32_t FixedMOD1 = 16777216;

typedef enum ADFType {
	ADF4355 = 0,
	ADF5355 = 1
} ADFType;

typedef enum ADFError {
	ADF_FREQERROR = 0
} ADFError;

typedef struct ADFParam {
	uint64_t freq,
	uint8_t divOut,
	uint64_t INT,
	uint64_t MOD1,
	uint64_t FRAC1,
	uint64_t MOD2,
	uint64_t FRAC2,
	uint8_t outx2
} ADFParam;

typedef struct ADFRegs {
	uint32_t R0,
	uint32_t R1,
	uint32_t R2,
	uint32_t R3,
	uint32_t R4,
	uint32_t R5,
	uint32_t R6,
	uint32_t R7,
	uint32_t R8,
	uint32_t R9,
	uint32_t R10,
	uint32_t R11,
	uint32_t R12
} ADFRegs;

class ADFX355
{
	public:
		ADFX355(uint64_t tcxo, uint64_t VCOMIN, uint64_t VCOMAX, uint64_t pfdTarget, ADFType type);
		void initFreqRange(uint64_t startFreq, uint64_t stopFreq, uint64_t nbPoints);
		
		uint64_t getCalcFreq(ADFParam * settings);
		
		ADFError computeParams(ADFParam * settings);
		
	private:
		uint64_t tcxo;
		uint64_t VCOMIN;
		uint64_t VCOMAX;
		uint64_t PFD;
		uint64_t R;
		uint8_t div2 = 0;
		uint8_t mul2 = 0;
		ADFType type = 0;
		
		ADFParam * freqSettings;
		
		uint64_t nbPoints;
};

#endif