% pll_time2.m 6/3/16 nr
% Digital PLL model in time using difference equations.
% fn = 5 kHz NCO initital freq error = -100ppm*8 MHz = -800 Hz
N= 30000; % number of samples
fref = 8e6; % Hz freq of ref signal
fs= 25e6; % Hz sample rate
Ts = 1/fs; % s sample time
n= 0:N-1; % time index
t= n*Ts*1000; % ms
init_phase = 0.7; % cycles initial phase of reference signal
ref_phase = fref*n*Ts + init_phase; % cycles phase of reference signal
ref_phase = mod(ref_phase,1); % cycles phase mod 1
Knco= 1/4096; % NCO gain constant
KI= .0032; % looop filter integrator gain
KL= 5.1; % loop filter linear (proportional) gain
fnco = fref*(1-100e-6); % Hz NCO initial frequency
u(1) = 0;
int(1)= 0;
phase_error(1) = -init_phase;
vtune(1) = -init_phase*KL;
% compute difference equations
for n= 2:N
    % NCO
    x = fnco*Ts + u(n-1) + vtune(n-1)*Knco; % cycles NCO phase
    u(n) = mod(x,1); % cycles NCO phase mod 1
    s = sin(2*pi*u(n-1)); % NCO sine output
    y(n)= round(2^15*s)/2^15; % quantized sine output
    % Phase Detector
    pe= ref_phase(n-1) - u(n-1); % phase error
    pe= 2*(mod(pe+1/2,1) - 1/2); % wrap if phase crosses +/- 1/2 cycle
    phase_error(n) = pe;
    % Loop Filter
    int(n) = KI*pe + int(n-1); % integrator
    vtune(n) = int(n) + KL*pe; % loop filter output
end
plot(t,phase_error),grid
axis([0 0.5 -1 1])
xlabel('t (ms)'),ylabel('phase error'),figure
plot(t,vtune),grid
axis([0 0.5 -3.5 1])
xlabel('t (ms)'),ylabel('vtune'),figure
psd(y(11000:end),2^14,fs/1e6)
axis([7.95 8.05 -80 40]),xlabel('MHz')