% Parse data dumped by the VNARefit
clear;

startingpoint = 0;
nbPoints = 250;
measures = zeros(nbPoints,2);
freqdetect = zeros(nbPoints,1);
maxamp = zeros(nbPoints,1);
load_name = "LOAD_20MHZ";
open_name = "OPEN_20MHZ";
short_name = "SHORT_20MHZ";
meas = "INREF_METAL";

startFreq = 7*1e9;
stopFreq = 13.5*1e9;

sampfreq = 20e6;

step = (stopFreq-startFreq) / (nbPoints - 1);

xaxis = (startFreq:step:stopFreq);
fftxaxis = (0:sampfreq/4096:sampfreq-1);

sigtab = zeros(1,nbPoints);
reftab = zeros(1,nbPoints);
sigph = zeros(1,nbPoints);
refph = zeros(1,nbPoints);
derror = zeros(1,nbPoints);
refpeak = zeros(1,nbPoints);
sigpeak = zeros(1,nbPoints);

for i = startingpoint:nbPoints-1
    % Calibration calculation
    load_path = [ "..\..\..\..\vnadump\S11_" , load_name , "\" , int2str(i), ".csv" ];
    open_path = [ "..\..\..\..\vnadump\S11_" , open_name , "\" , int2str(i), ".csv" ];
    short_path = [ "..\..\..\..\vnadump\S11_" , short_name , "\" , int2str(i), ".csv" ];
    load = dlmread(strjoin(load_path,''), ";");
    open = dlmread(strjoin(open_path,''), ";");
    short = dlmread(strjoin(short_path,''), ";");
    
    lsig = load(:,2);
    lref = load(:,1);
    
    [lsiginf,lrefinf,lcref] = getSigInfo(lsig, lref);
    
    osig = open(:,2);
    oref = open(:,1);
    
    [osiginf,orefinf,ocref] = getSigInfo(osig, oref);
    
    ssig = short(:,2);
    sref = short(:,1);
    
    [ssiginf,srefinf,scref] = getSigInfo(ssig, sref);
    
    % DFT signal
    meas_path = [ "..\..\..\..\vnadump\S11_" , meas , "\" , int2str(i), ".csv" ];
    signals = dlmread(strjoin(meas_path,''), ";");
    
    tsig = signals(1:end,2);
    tref = signals(1:end,1);
    
    [sig, ref, cref] = getSigInfo(tsig, tref);
    
    % Sig Calibration
    [a,b,c] = cal_coef(ssiginf/srefinf, osiginf/orefinf, lsiginf/lrefinf);
    
    derror(i+1) = a;
    
    %S11 = sig / ref; 
    S11 = (a - c * (sig/ref))/(b * (sig/ref) - 1);
    
    % Save point
    reftab(i+1) = abs(ref);
    sigtab(i+1) = abs(S11);
    
    refph(i+1) = atan2(imag(ref), real(ref));
    sigph(i+1) = atan2(imag(S11), real(S11));
    
    refpeak(i+1) = (max(tref)+abs(min(tref)))/2;
    sigpeak(i+1) = (max(tsig)+abs(min(tsig)))/2;
    
%     subplot(2,1,1);
%     plot(fftxaxis,mag2db(abs(fft(tref/65536)/4096)));
%     hold on;
%     subplot(2,1,2);
%     plot(fftxaxis,mag2db(abs(fft(tsig/65536)/4096)));
%     hold on;
    
end

figure;
hold on;
yyaxis left;
plot(xaxis,refpeak/65536*1000);
title("Ref V Peak");
xlabel("Hz"); ylabel("mV");
grid on;
grid minor;
yyaxis right;
plot(xaxis,refpeak);
ylabel("ADC Code");


figure;
hold on;
yyaxis left;
plot(xaxis,sigpeak/65536*1000);
title("Signal V Peak");
xlabel("Hz"); ylabel("mV");
grid on;
grid minor;
yyaxis right;
plot(xaxis,sigpeak);
ylabel("ADC Code");

% figure;
% subplot(2,1,1);
% hold on;
% plot(xaxis,mag2db(sigtab));
% title("S11 Amplitude");
% xlabel("Hz"); ylabel("dB");
% grid on;
% grid minor;
% 
% subplot(2,1,2);
% hold on;
% plot(xaxis,wrapTo180(rad2deg(sigph)));
% title("S11 Phase");
% xlabel("Hz"); ylabel("Degrees -180/180");
% grid on;
% grid minor;

% testsig = hilbert(signals(:,2)/65535);
% testref = hilbert(signals(:,1)/65535);
% testref(1) = 0;
% 
% refsig = hilbert(load(:,2)/65535);
% refref = hilbert(load(:,1)/65535);
% refref(1) = 0;

%figure;
%subplot(2,2,1);
% plot(fftxaxis,mag2db(abs(fft(signals(:,1)/65535)/1000)));
% hold on;
% plot(fftxaxis,mag2db(abs(fft(signals(:,2)/65535)/1000)));
% subplot(2,2,2);
% plot(fftxaxis,mag2db(abs(fft(load(:,1)/65535)/1000)));
% hold on;
% plot(fftxaxis,mag2db(abs(fft(load(:,2)/65535)/1000)));
% subplot(2,2,3);
% plot(fftxaxis,mag2db(abs(fft(short(:,1)/65535)/1000)));
% hold on;
% plot(fftxaxis,mag2db(abs(fft(short(:,2)/65535)/1000)));
% subplot(2,2,4);
% plot(fftxaxis,mag2db(abs(fft(open(:,1)/65535)/1000)));
% hold on;
% plot(fftxaxis,mag2db(abs(fft(open(:,2)/65535)/1000)));


%subplot(2,2,3);
%polarplot(S11);
