#ifndef ADF4355_H
#define ADF4355_H

#include <stdint.h>
#include <numeric>
#include "general.h"

#define AUTOCALEN(a) (a | 0x00200000)
#define AUTOCALDIS(a) (a & ~0x00200000)

#define CNTRSTEN(a) (a | 0x00000010)
#define CNTRSTDIS(a) (a & ~0x00000010)

class ADFParam
{
private:
    uint32_t regs[13];

    uint8_t blob[14][5];
    uint8_t upblob[9][5];

    uint8_t fblob[70];
    uint8_t fupblob[45];

    uint8_t divOut;
    uint64_t INT;
    uint64_t MOD1;
    uint64_t FRAC1;
    uint64_t MOD2;
    uint64_t FRAC2;
    uint8_t div2 = 0;
    uint8_t mul2 = 0;
    uint64_t R;
    uint64_t PFD;

    uint64_t channel;

    uint8_t upperOut = 0;

public:
    ADFParam();
    ADFParam(uint32_t freq);
    ADFParam(uint32_t freq, uint32_t *regs);

    void setBlobs(uint8_t type);
    uint8_t * getBlob();
    uint8_t * getUpdateBlob();

    void setReg(uint8_t regN, uint32_t regVal);
    uint32_t getReg(uint8_t regN);
    void setFreq(uint32_t freq);
    void setRegs(uint8_t type);

    void FreqParams(uint64_t freq, uint64_t VCOMIN, uint64_t VCOMAX, uint32_t PFD, uint8_t div2, uint8_t mul2, uint16_t R);

    void setChannel(uint64_t step);
    uint64_t getCalcFreq();
    bool isFreqUpper();

    uint64_t freq;
};

class ADF4355
{
public:
    ADF4355(uint32_t tcxo, uint64_t VCOMIN, uint64_t VCOMAX, uint32_t pfdTarget, uint8_t type);
    void initFreqRange(uint64_t startFreq, uint64_t stopFreq, uint16_t nbPoints);
    ADFParam * FreqParams(uint64_t freq);

    uint8_t * getBytes(uint16_t pointN);
    uint8_t * getUpdateBytes(uint16_t pointN);

    ADFParam * freqParams;
    uint64_t *freqs;

private:
    uint32_t tcxo;
    uint64_t VCOMIN;
    uint64_t VCOMAX;
    uint64_t PFD;
    uint64_t R;
    uint8_t div2 = 0;
    uint8_t mul2 = 0;
    uint8_t type = 0;
};

#endif // ADF4355_H
