#include "fft_wrap.h"

/*
 * Store the base plan
 */
bool fftwInit = false;
fftw_plan defaultFFT;
fftw_plan defaultIFFT;

void analytic(fftw_complex *in, uint16_t size, bool dcrem) {
    if(!fftwInit) {
        // We sacrifice the first measure to optimize FFT
        fftw_complex *dumPad = (fftw_complex*)fftw_malloc(sizeof(fftw_complex)*FFTSIZE);
        defaultFFT = fftw_plan_dft_1d(FFTSIZE, dumPad, dumPad, FFTW_FORWARD, FFTW_MEASURE);
        defaultIFFT = fftw_plan_dft_1d(FFTSIZE, dumPad, dumPad, FFTW_BACKWARD, FFTW_MEASURE);
        fftw_free(dumPad);
        fftwInit = true;
    }

    /*
     * new-array execute functions
     */
    fftw_execute_dft(defaultFFT, in, in);

    /*
     * Can be optimized !
     * We put complex part of spectrum at 0 and multiply by 2 the real part
     * There is a slight difference when FFT size is odd or even:
     * https://fr.mathworks.com/help/matlab/ref/fftshift.html
     */

#if FFTSIZE % 2 == 0
    #define MID (FFTSIZE/2-1)
#else
    #define MID (FFTSIZE/2)
#endif

    if(!dcrem) {
        in[0][0] = in[0][0]/(double)FFTSIZE;
        in[0][1] = in[0][1]/(double)FFTSIZE;
    } else {
        in[0][0] = 0;
        in[0][1] = 0;
    }
    for(uint16_t i = 1; i < FFTSIZE; i++) {
        if(i < MID) {
            if(i < 510 && i > 310) { // Crappy Filtering need a sinc
                in[i][0] = in[i][0]*2;
                in[i][1] = in[i][1]*2;
                in[i][0] = in[i][0]/(double)FFTSIZE;
                in[i][1] = in[i][1]/(double)FFTSIZE;
            } else {
                in[i][0] = 0;
                in[i][1] = 0;
            }
        } else if (i == MID) {
            in[i][0] = in[0][0]/(double)FFTSIZE;
            in[i][1] = in[0][1]/(double)FFTSIZE;
        } else {
            in[i][0] = 0;
            in[i][1] = 0;
        }
    }

    fftw_execute_dft(defaultIFFT, in, in);

    /* Not sure why it's normalized after the IFFT */
//    for(uint16_t i = 0; i < FFTSIZE; i++) {
//        in[i][0] = in[i][0]/FFTSIZE;
//        in[i][1] = in[i][1]/FFTSIZE;
//    }
}

const double pi = std::acos(-1);
const std::complex<double> i(0, 1);
std::complex<double> pllout[FFTSIZE];

double pll(std::complex<double> * in, std::complex<double> *& out, uint16_t size) {
    // Fast PLL see: http://liquidsdr.org/blog/pll-simple-howto/
    double phasecoef = 0.08;
    double freqcoef = 0.05 * phasecoef * phasecoef;

    std::complex<double> error;
    double argerr;
    double pcorr = 0;
    double fcorr = 0;

    for(uint16_t k = 0; k < FFTSIZE; k++) {
        pllout[k] = std::exp(i*(pcorr+fcorr));
        error = std::conj(pllout[k]) * in[k];

        argerr = std::atan2(std::imag(error), std::real(error));

        pcorr = pcorr + phasecoef * argerr;
        fcorr = fcorr + freqcoef * argerr;
        pcorr = pcorr + fcorr;
    }

    out = pllout;
    return fcorr;
}

/*
 * This function compute the 1 point convolution between two signals of the same size.
 */
std::complex<double> convol1pt(std::complex<double> * ref, std::complex<double> * sig, uint16_t start_offset = 0) {
    std::complex<double> convol (0,0);

    for(uint16_t k = start_offset; k < FFTSIZE; k++) {
        convol += ref[k] * sig[FFTSIZE-1-k+start_offset];
    }

    return convol/((double)FFTSIZE-(double)start_offset);
}

double mag2db(double mag) {
    return (double)20 * std::log10(mag);
}
