#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QPainter>
#include <complex>

#include "marker.h"
#include "ftdiwrap.h"
#include "adf4355.h"
#include "vnathread.h"
#include "fft_wrap.h"
#include "smithwin.h"
#include "mysmithwidget.h"

//#include "dsp.h"

#define MEASURE         0
#define SHORT           1
#define OPEN            2
#define LOAD            3

#define MAXMARKERS      16

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    double convertToValues(const QString& input);
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

public slots:
    void getData(const DataVector *c1);
    void setMarker(const QPointF *p);

protected:
    Marker * markers;
    uint16_t curMarker = 0;

    bool refreshMarkers = false;

private slots:
    void on_bConnect_clicked();

    void on_bStart_clicked();

    void on_bStop_clicked();

    void on_bShort_clicked();

    void on_bOpen_clicked();

    void on_bLoad_clicked();

    void on_bSmith_clicked();

private:
    Ui::MainWindow *ui;
    FTDI *vnaFTDI;
    ADF4355 *pllRef;
    ADF4355 *pllGen;
    VNAThread *myVNA;
	
    uint64_t nbPoints = 0;
    uint8_t mode = 0;
    uint8_t calstatus = 0;
    bool shortcalval = false;
    bool opencalval = false;
    bool loadcalval = false;
    std::complex<double> *shortcal = 0;
    std::complex<double> *opencal = 0;
    std::complex<double> *loadcal = 0;
	uint16_t measIndex = 0;

    SmithWin *smithwin;
    bool smithHide = true;
    MySmithWidget *scwid = 0;

    uint32_t markCnt = 0;
    uint32_t markList[MAXMARKERS];
};

#endif // MAINWINDOW_H
