#include "myglwidget.h"

MyGLWidget::MyGLWidget(QWidget *parent) : QOpenGLWidget(parent)
{
    setAutoFillBackground(false);
}

void MyGLWidget::setSize(uint16_t size) {
    if(mag != 0) {
        delete[] mag;
        delete[] ang;
    }

    this->size = size;
    mag = new QPointF[size];
    ang = new QPointF[size];
    refmag = new QPointF[size];
}

void MyGLWidget::mouseMoveEvent(QMouseEvent *event) {

}

void MyGLWidget::paintEvent(QPaintEvent *event) {
    QPainter painter;
    painter.begin(this);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.fillRect(this->rect(), QBrush(QColor(0, 0, 0)));

    double ysize = this->height();
    double ydiv = ysize/(double)200;
    double xsize = this->width();

    // Y symmetry
    QMatrix m;
    m.translate(0, ysize);
    m.scale(1, -1);
    painter.setMatrix(m);

    // Draw data
    painter.setPen(QPen(QColor(255,0,0)));
    painter.drawPolyline(mag, this->size);
    painter.setPen(QPen(QColor(0,255,0)));
    painter.drawPolyline(ang, this->size);
    painter.setPen(QPen(QColor(0,0,255)));
    painter.drawPolyline(refmag, this->size);

    // Draw Grid
    painter.setPen(QPen(QColor(127,127,127)));

    QPointF hgrid[40];
    uint8_t l = 0;
    for(uint16_t k = 0; k < 20; k += 2) {
        hgrid[k].setX(0);
        hgrid[k].setY(ydiv*10*k);
        hgrid[k+1].setX(xsize);
        hgrid[k+1].setY(ydiv*10*k);
        painter.setWorldMatrixEnabled(false);
        painter.drawText(hgrid[k], QString::number(100-l*20));
        painter.setWorldMatrixEnabled(true);
        l++;
    }
    painter.drawLines(hgrid, 20);

    painter.save();

    painter.end();
}

void MyGLWidget::mousePressEvent(QMouseEvent *event) {
    QPointF *c = new QPointF(event->pos());
    emit sendMarkPoint(c);
}
