#include "vnathread.h"

#include <string>
#include <iostream>
#include <fstream>

#include <QElapsedTimer>

DataVector::DataVector() {

}

DataVector::DataVector(uint16_t size, uint16_t fPoint) {
    //this->sigDut = new std::complex<double>[size];
    //this->sigRef = new std::complex<double>[size];
    // Malloc in C++ ?
    this->sigRef = (fftw_complex*)fftw_malloc(sizeof(fftw_complex) *size);
    this->sigDut = (fftw_complex*)fftw_malloc(sizeof(fftw_complex) *size);
    this->fPoint = fPoint;
}

DataVector::~DataVector() {
    //delete [] this->sigDut;
    //delete [] this->sigRef;
    fftw_free(this->sigRef);
    fftw_free(this->sigDut);
}

VNAThread::VNAThread(QObject *parent) : QThread(parent)
{
    abort = false;
    this->vna = vna;
    this->size = size;
}

VNAThread::~VNAThread() {
    mutex.lock();
    abort = true;
    condition.wakeOne();
    mutex.unlock();

    wait();
}

void VNAThread::setParam(uint16_t size, FTDI *vna, ADF4355 *pllParam, ADF4355 *genParam, uint16_t nbPoints) {
    this->size = size;
    this->vna = vna;
    this->pllParam = pllParam;
    this->genParam = genParam;
    this->nbPoints = nbPoints;
}

void VNAThread::stopProcess() {
    this->abort = true;
}

void VNAThread::resetPLLs() {
    uint8_t PLLRst[] = {CONFIGREF_CMD, 0x00, 0x00, 0x00, 0x44};
    DWORD tmpw;
    this->vna->sendData(PLLRst, 5, &tmpw);
    PLLRst[0] = CONFIGGEN_CMD;
    this->vna->sendData(PLLRst, 5, &tmpw);
}

void VNAThread::setPLLs(uint16_t i) {
    // Send Pll parameters
    DWORD wrote;
    uint64_t data_len;
    uint8_t * data;
    if(this->initPLL) {
        data = pllParam->getBytes(i);
        data_len = 70;
    } else {
        data = pllParam->getUpdateBytes(i);
        data_len = 45;
    }

    this->vna->sendData(data, data_len, &wrote);

    // Send PLL parameters for teh generator
    if(this->initPLL) {
        data = genParam->getBytes(i);
        data_len = 70;
    } else {
        data = genParam->getUpdateBytes(i);
        data_len = 45;
    }
    this->vna->sendData(data, data_len, &wrote);
}

void VNAThread::run() {
    uint16_t i = 0;

    uint8_t samples[32769];

    // Start Mixer and put relay in low range
    DWORD analogWrite = 0;
    uint8_t analogCtrl[2] = { 0x6, 0x13 };
    this->vna->sendData(analogCtrl, 2, &analogWrite);
    msleep(20);

    bool lowRange = true; // Used to wait the relay swithing
    bool wait = false;

    // Configure the ADC !
    DWORD adcwrote = 0;
    uint8_t ADCregs[] = { 0x03, 0x00, 0x80, 0x03, 0x02, 0x01, 0x03, 0x03, 0xFF, 0x03, 0x04, 0xFF, 0x03, 0x01, 0x20 };
    this->vna->sendData(ADCregs, 15, &adcwrote);

    QElapsedTimer dbgTimer;
    qint64 dbgnsec;

    forever {
        // Bad mutex lock but nobody should
        // access the FTDI

        // Set the X2 input of mixer
        if(pllParam->freqParams[i].isFreqUpper()) {
            analogCtrl[1] = 0xF; // We activate the two x2 mixer
        } else {
            analogCtrl[1] = 0x3;
        }
        if(genParam->freqParams[i].isFreqUpper()) {
            analogCtrl[1] |= 0x20;
            wait = false;
            if(lowRange) {
                lowRange = false;
                wait = true;
            }
        } else {
            analogCtrl[1] |= 0x10;
            wait = false;
            if(!lowRange) {
                lowRange = true;
                wait = true;
            }
        }
        this->vna->sendData(analogCtrl, 2, &analogWrite);

        if(wait) msleep(20);

        this->setPLLs(i);

        // We wait here for the PLL lock
        // Sometime lock fail (don't know why for now) so we will reset the PLL
        // 0: PLLREF LOCKED
        // 1: PLLGEN LOCKED
        uint8_t status[1] = { 0 };
        DWORD wrote;
        DWORD toread = 0;
        DWORD red = 0;
        uint8_t statusCmd[1] = { 0x7 };
        uint8_t nbWait = 25; // Number try before resetting PLL ! (25*10ms because 110 ms is the mean waiting time value)
        uint8_t wcount = 0;

        //dbgTimer.restart();
        do {
            this->vna->sendData(statusCmd, 1, &wrote);
            msleep(1);
            do {
                this->vna->getNbBytes(&toread);
            } while(toread == 0);
            this->vna->readData(status, toread, &red);
            //qDebug("Lock Status: %d", status[0]);
            wcount++;
            if(wcount == nbWait-1) {
                msleep(10);
                qDebug("Number of wait overflown");
                this->resetPLLs();
                // Should I reset the first config indicator ?
                this->setPLLs(i);
                wcount = 0;
            }
        } while((status[0] & 0x3) != 3);
        //dbgnsec = dbgTimer.nsecsElapsed();
        //qDebug( "number of wait cycle is: %d", wcount );
        //qDebug("number of msec for lock: %f", (float)dbgnsec/(float)1e6);

        // Get samples
        uint16_t scount  = 0;

        uint8_t getSampCMD[] = {0x04};

        this->vna->sendData(getSampCMD, 1, &wrote);
        // Queue status function crash if called too soon !
        // TBF
        msleep(1);

        //dbgTimer.restart();
        while(scount < 32768) {
            this->vna->getNbBytes(&toread);
            if(toread > 0) {
                this->vna->readData(&samples[scount], toread, &red);
                scount = scount + red;
            }
        }
        //dbgnsec = dbgTimer.nsecsElapsed();
        //qDebug("number of msec for datagrab: %f", (float)dbgnsec/(float)1e6);

        DataVector * dv = new DataVector(8192, i);
        uint16_t j = 0;
        for(j = 0; j < 32768; j+=4) {
            dv->sigDut[j/4][0] = (double)((int16_t)(samples[j] << 8) | (int16_t)(samples[j+1]));
            dv->sigRef[j/4][0] = (double)((int16_t)(samples[j+2] << 8) | (int16_t)(samples[j+3]));
        }

        // Emit samples
        emit sendData(dv);

        // Check abort
        if(this->abort) {
            i = 0;
            this->abort = 0;
            return;
        }

        // PLL already used
        this->initPLL = false;

        // Debug crap
//        std::string path = "C:\\Users\\nats\\Documents\\vna_dump\\" + std::to_string(i) + ".csv";
//        std::ofstream file;

//        file.open(path, std::ios_base::out);
//        for(uint16_t j = 0; j  < 4096; j++) {
//            file << dv->sigRef[j][0] << ";" << dv->sigDut[j][0] << std::endl;
//        }

//        file.close();

        // Increment point
        i++;

        // For debug
        if(i==nbPoints) {
            return;
            //i = 0;
        }
    }
}
