#ifndef VNATHREAD_H
#define VNATHREAD_H

#include <QThread>
#include <QMutex>
#include <QWaitCondition>

#include <complex>

#include "ftdiwrap.h"
#include "adf4355.h"
#include "fftw-3.3.5-64/fftw3.h"

class DataVector
{
public:
    DataVector();
    DataVector(uint16_t size, uint16_t fPoint);
    ~DataVector();

    //std::complex<double> * sigRef;
    //std::complex<double> * sigDut;
    fftw_complex *sigRef;
    fftw_complex *sigDut;
    uint16_t fPoint;
};

Q_DECLARE_METATYPE(DataVector)

class VNAThread : public QThread
{
    Q_OBJECT

public:
    VNAThread(QObject * parent = 0);
    ~VNAThread();
    void setParam(uint16_t size, FTDI * vna, ADF4355 *pllParam, ADF4355 *genParam, uint16_t nbPoints);
    void resetPLLs();
    void setPLLs(uint16_t i);

    ADF4355 *pllParam;
    ADF4355 *genParam;

public slots:
    void stopProcess();

protected:
    void run();

signals:
    void sendData(const DataVector * c1);

private:
    QMutex mutex;
    QWaitCondition condition;
    bool abort;
    uint16_t size;
    FTDI *vna;
    bool initPLL = true;
    uint16_t nbPoints;
};

#endif // VNATHREAD_H
