`default_nettype none

module adc_interface(
	clklogic,
	DCO_P, DCO_N, FR_P, FR_N,
	AD_1A_P, AD_1A_N, AD_1B_P, AD_1B_N,
    AD_1C_P, AD_1C_N, AD_1D_P, AD_1D_N,
    AD_2A_P, AD_2A_N, AD_2B_P, AD_2B_N,
    AD_2C_P, AD_2C_N, AD_2D_P, AD_2D_N,
	ENC_P, ENC_N,
	dout, dout_valid, dread, dbusy, dstart
	);
	
	input wire clklogic;
	
	input wire DCO_P;
	input wire DCO_N;
	
	input wire FR_P;
	input wire FR_N;
	
	input wire AD_1A_P;
	input wire AD_1A_N;
	input wire AD_1B_P;
	input wire AD_1B_N;
	input wire AD_1C_P;
	input wire AD_1C_N;
	input wire AD_1D_P;
	input wire AD_1D_N;
	
	input wire AD_2A_P;
	input wire AD_2A_N;
	input wire AD_2B_P;
	input wire AD_2B_N;
	input wire AD_2C_P;
	input wire AD_2C_N;
	input wire AD_2D_P;
	input wire AD_2D_N;
	
	output wire ENC_P;
	output wire ENC_N;
	
	output wire [7:0]dout;
	output wire dout_valid;
	(* ASYNC_REG = "TRUE" *)output reg dbusy = 0;
	input wire dread;
	input wire dstart;
	
	reg busy = 0;
	(* ASYNC_REG = "TRUE" *)reg busy_0 = 0;
	
	always @(posedge clklogic) 
	begin
		busy_0 <= busy;
		dbusy <= busy_0;
	end
	
	wire DCO;
	
	wire FR;
	
	wire AD_1A;
	wire AD_1B;
	wire AD_1C;
	wire AD_1D;
	
	wire AD_2A;
	wire AD_2B;
	wire AD_2C;
	wire AD_2D;
	
 	wire NDCO;
	IBUFGDS DCO_BUFDS(.I(DCO_N), .IB(DCO_P), .O(NDCO));
	assign DCO = ~NDCO;

	wire NFR;
	IBUFDS FR_BUFDS(.I(FR_N), .IB(FR_P), .O(NFR));
	assign FR = ~NFR;
	
	wire NAD_1A;
	IBUFDS AD_1AB(.I(AD_1A_N), .IB(AD_1A_P), .O(NAD_1A));
	assign AD_1A = ~NAD_1A;
	
	wire NAD_1B;
	IBUFDS AD_1BB(.I(AD_1B_N), .IB(AD_1B_P), .O(NAD_1B));
	assign AD_1B = ~NAD_1B;
	
	wire NAD_1C;
	IBUFDS AD_1CB(.I(AD_1C_N), .IB(AD_1C_P), .O(NAD_1C));
	assign AD_1C = ~NAD_1C;
	
	wire NAD_1D;
	IBUFDS AD_1DB(.I(AD_1D_N), .IB(AD_1D_P), .O(NAD_1D));
	assign AD_1D = ~NAD_1D;
	
	wire NAD_2A;
	IBUFDS AD_2AB(.I(AD_2A_N), .IB(AD_2A_P), .O(NAD_2A));
	assign AD_2A = ~NAD_2A;
	
	wire NAD_2B;
	IBUFDS AD_2BB(.I(AD_2B_N), .IB(AD_2B_P), .O(NAD_2B));
	assign AD_2B = ~NAD_2B;
	
	wire NAD_2C;
	IBUFDS AD_2CB(.I(AD_2C_N), .IB(AD_2C_P), .O(NAD_2C));
	assign AD_2C = ~NAD_2C;
	
	wire NAD_2D;
	IBUFDS AD_2DB(.I(AD_2D_N), .IB(AD_2D_P), .O(NAD_2D));
	assign AD_2D = ~NAD_2D;
	
	/* ENC output */
	wire clk20, adc_pll_out;
	wire clkfb;
	
	PLL_BASE #(
		.BANDWIDTH("OPTIMIZED"),
		.CLK_FEEDBACK("CLKFBOUT"),
		.CLKFBOUT_MULT(20),
		.DIVCLK_DIVIDE(1),
		.CLKOUT0_DIVIDE(24),
		.CLKIN_PERIOD(41.6)
	) pll_outgen (
		.CLKIN(clklogic),
		.RST(1'b0),
		.CLKFBIN(clkfb),
		
		.CLKFBOUT(clkfb),
		
		.CLKOUT0(adc_pll_out),
		.CLKOUT1(),
		.CLKOUT2(),
		.CLKOUT3(),
		.CLKOUT4(),
		.CLKOUT5()
	);
	
	BUFG clk20_bufg(.I(adc_pll_out), .O(clk20));
	
	wire ENCDDR;
	
	ODDR2 #(.DDR_ALIGNMENT("C0"), .INIT(0), .SRTYPE("ASYNC")) 
			ENCDDRO(.Q(ENCDDR), .C0(clk20), .C1(~clk20), .CE(1),
					.D0(0), .D1(1), .R(0), .S(0));
	
	OBUFDS ENCBUFDS(.I(ENCDDR), .O(ENC_P), .OB(ENC_N));
	
	/* DCO Input and Framing sync */
	wire clk80, adc_pll_in;
	wire clk20_in, clk20_pll_in;
	wire clkfbin;
	
	PLL_BASE #(
		.BANDWIDTH("OPTIMIZED"),
		.CLK_FEEDBACK("CLKFBOUT"),
		.CLKFBOUT_MULT(12),
		.CLKOUT0_DIVIDE(6),
		.CLKOUT1_DIVIDE(24),
		.CLKIN_PERIOD(25)
	) pll_ingen (
		.CLKIN(DCO),
		.RST(1'b0),
		.CLKFBIN(clkfbin),
		
		.CLKFBOUT(clkfbin),
		
		.CLKOUT0(adc_pll_in),
		.CLKOUT1(clk20_pll_in),
		.CLKOUT2(),
		.CLKOUT3(),
		.CLKOUT4(),
		.CLKOUT5()
	);
	
	BUFG clk80_bufg(.I(adc_pll_in), .O(clk80));
	BUFG clk20in_bufg(.I(clk20_pll_in), .O(clk20_in));
	
	(* ASYNC_REG = "TRUE" *)reg start_0 = 0;
	(* ASYNC_REG = "TRUE" *)reg start_record = 0;
	always @(posedge clk80)
	begin
		start_0 <= dstart;
		start_record <= start_0;
	end
	
	// Async FIFO as storage
	
	reg [7:0]adc_data = 8'd0;
	reg wr_samp = 0;
	wire store_empty;
	wire full_thresh;
	
	adc_fifo_chan1 chan1_store (
		.rst(0), // input rst
		.wr_clk(clk80), // input wr_clk
		.rd_clk(clklogic), // input rd_clk
		.din(adc_data), // input [15 : 0] din
		.wr_en(wr_samp), // input wr_en
		.rd_en(dread), // input rd_en
		.prog_full_thresh(15'd32767), // input [9 : 0] prog_full_thresh
		.dout(dout), // output [15 : 0] dout
		.full(), // output full
		.empty(store_empty), // output empty
		.valid(dout_valid), // output valid
		.prog_full(full_thresh) // output prog_full
	);
	
	/* FR is used to sync the start of frame
	 * 01: start of frame
	 * 11: 2
	 * 10: 3
	 * 00: 4
	 * Need to lag by one cycle
	 */
	reg [1:0]fr_prev = 2'b00;
	reg [15:0]adc_data_1_80 = 16'd0;
	reg [15:0]adc_data_2_80 = 16'd0;
	
	reg [15:0]adc_data_1_20 = 16'd0;
	reg [15:0]adc_data_2_20 = 16'd0;
	
	always @(posedge clk80)
	begin
		fr_prev <= {fr_prev[0], FR};
		case(fr_prev)
			2'b00: begin // Actual start of frame
				adc_data_1_80[15:14] <= { AD_1A, AD_1B};
				adc_data_2_80[15:14] <= { AD_2A, AD_2B};
				adc_data_1_80[7:6] <= { AD_1C, AD_1D};
				adc_data_2_80[7:6] <= { AD_2C, AD_2D};
				// Store data to split it in Byte
				adc_data_1_20 <= adc_data_1_80;
				adc_data_2_20 <= adc_data_2_80;
				// Send nibble
				adc_data <= adc_data_1_80[15:8];
				// Start logging
				if(!start_record && full_thresh) begin
					wr_samp <= 0;
					busy <= 0;
				end else if(start_record && store_empty) begin
					wr_samp <= 1;
					busy <= 1;
				end
			end
			2'b01: begin
				adc_data_1_80[13:12] <= { AD_1A, AD_1B};
				adc_data_2_80[13:12] <= { AD_2A, AD_2B};
				adc_data_1_80[5:4] <= { AD_1C, AD_1D};
				adc_data_2_80[5:4] <= { AD_2C, AD_2D};
				
				adc_data <= adc_data_1_20[7:0];
			end
			2'b11: begin
				adc_data_1_80[11:10] <= { AD_1A, AD_1B};
				adc_data_2_80[11:10] <= { AD_2A, AD_2B};
				adc_data_1_80[3:2] <= { AD_1C, AD_1D};
				adc_data_2_80[3:2] <= { AD_2C, AD_2D};
				
				adc_data <= adc_data_2_20[15:8];
			end
			2'b10: begin
				adc_data_1_80[9:8] <= { AD_1A, AD_1B};
				adc_data_2_80[9:8] <= { AD_2A, AD_2B};
				adc_data_1_80[1:0] <= { AD_1C, AD_1D};
				adc_data_2_80[1:0] <= { AD_2C, AD_2D};
				
				adc_data <= adc_data_2_20[7:0];
			end
		endcase
	end
	
endmodule
