`timescale 1ns / 1ps
`default_nettype none

module adc_storage(
		clkadc, chan1, data_sync,
		clklogic, oready, ovalid,
		odata1,
		nbsample, start
    );
	
	input wire clkadc;
	input wire [7:0]chan1;
	input wire data_sync;
	
	input wire clklogic;
	input wire oready;
	output wire ovalid;
	
	output wire [7:0]odata1;
	
	input wire [12:0]nbsample;
	input wire start;
	
	
	
	reg rst_store = 1;
	reg store_data_en = 0, start_sync = 0;
	wire store_valid;
	wire rd_done;
	wire store_empty;
	
	(* ASYNC_REG = "TRUE" *)reg w_en_0 = 0;
	(* ASYNC_REG = "TRUE" *)reg w_en = 0;
	
	always @(posedge clkadc) 
	begin
		w_en_0 <= store_data_en & start_sync;
		w_en <= w_en_0;
	end
	
	adc_fifo_chan1 chan1_store (
		.rst(rst_store), // input rst
		.wr_clk(clkadc), // input wr_clk
		.rd_clk(clklogic), // input rd_clk
		.din(chan1), // input [15 : 0] din
		.wr_en(w_en), // input wr_en
		.rd_en(oready), // input rd_en
		.prog_full_thresh(nbsample), // input [9 : 0] prog_full_thresh
		.dout(odata1), // output [15 : 0] dout
		.full(), // output full
		.empty(store_empty), // output empty
		.valid(store_valid), // output valid
		.prog_full(rd_done) // output prog_full
	);
	
	// FSM to handle the read operation
	// We read until the rd_done trigger
	reg data_ok = 0;
	
	assign ovalid = store_valid & data_ok;
	
	localparam IDLE		= 3'd0;
	localparam READADC	= 3'd1;
	localparam VALIDDATA	= 3'd2;
	
	reg [2:0]state = IDLE;
	
	always @(posedge clklogic)
	begin
		case(state)
			IDLE: begin
				if(start) begin
					state <= READADC;
				end
			end
			READADC: begin
				if(rd_done) begin
					state <= VALIDDATA;
					start_sync <= 0;
				end else if(data_sync == 1) begin
					start_sync <= 1;
				end
			end
			VALIDDATA: begin
				if(store_empty) begin
					state <= IDLE;
				end
			end
		endcase
	end
	
	always @(*)
	begin
		store_data_en <= 0;
		data_ok <= 0;
		rst_store <= 0;
		case(state)
			IDLE: begin
				rst_store <= 1;
			end
			READADC: begin
				store_data_en <= 1;
			end
			VALIDDATA: begin
				data_ok <= 1;
			end
		endcase
	end
endmodule
