`timescale 1ns / 1ps
`default_nettype none

module ft2232_ft245(
		FT_CLK, NRXF, NTXE, NRD, NWR, NOE, SIWU, FDATA, // FTDI Interface
		DIN, IVALID, IREADY, // Input data from USB
		DOUT, OVALID, OREADY // Output data to USB
    );

	input wire FT_CLK;
	input wire NRXF;
	input wire NTXE;
	
	output wire NRD;
	output wire NWR;
	output wire NOE;
	output wire SIWU;
	
	inout wire [7:0]FDATA;
	
	output wire [7:0]DIN;
	output wire IVALID;
	input wire IREADY;
	
	input wire [7:0]DOUT;
	input wire OVALID;
	output reg OREADY;
	
	assign SIWU = 1;
	
	wire RXF = !NRXF;
	wire TXE = !NTXE;
	
	reg RD = 0;
	reg WR = 0;
	reg OE = 0;
	
	assign NRD = !RD;
	assign NWR = !(WR);
	assign NOE = !OE;
	
	wire [7:0]FDATA_IN = FDATA;
	wire [7:0]FDATA_OUT;
	assign FDATA = OE ? 8'dZ : FDATA_OUT;
	
	assign DIN = FDATA_IN;
	assign FDATA_OUT = DOUT;
	
	wire FT_READ = RXF && IREADY;
	wire FT_WRITE = TXE && OVALID;
	
	reg IVAL = 0;
	assign IVALID = IVAL & RXF; // We need to and the signal with the RXF signal to stop rx soon enough
	
	localparam IDLE 		= 0;
	localparam PRE_WRITE	= 1;
	localparam WRITE		= 2;
	localparam PRE_READ	= 3;
	localparam READ		= 4;
	
	reg [3:0]state = IDLE;
	
	reg [3:0]count = 0;
	
	always @(posedge FT_CLK)
	begin
		
		OE <= 0;
		RD <= 0;
		
		IVAL <= 0;
		
		count <= 0;
		
		case(state)
			IDLE: begin
				if(FT_READ) begin
					state <= PRE_READ;
				end else if(FT_WRITE) begin
					state <= PRE_WRITE;
				end
			end
			
			PRE_READ: begin
				if(FT_READ) begin
					OE <= 1;
					count <= 1;
					if(count == 1) state <= READ;
				end else begin
					state <= IDLE;
				end
			end
			
			READ: begin
				if(FT_READ) begin
					OE <= 1;
					RD <= 1;
					IVAL <= 1;
				end else begin
					state <= IDLE;
				end
			end
			
			PRE_WRITE: begin
				if(FT_WRITE) begin
					state <= WRITE;
				end else begin
					state <= IDLE;
				end
			end
			
			WRITE: begin
				if(FT_WRITE) begin
				end else begin
					state <= IDLE;
				end
			end
		endcase
	end
	
	always @(*)
	begin
		OREADY = 0;
		WR = 0;
		
		case(state)
			WRITE: begin
				if(FT_WRITE) begin
					OREADY = 1;
					WR = 1;
				end
			end
		endcase
	end
	
//	wire [35:0]CONTROL0;
//
//	chipscope_icon chipcon(.CONTROL0(CONTROL0));
//
//	chipscope_ila rxftdi (
//		.CONTROL(CONTROL0),
//		.CLK(FT_CLK),
//		.DATA({RD,IREADY, RXF,IVALID,IREADY,DOUT}),
//		.TRIG0(state == READ)
//	);

endmodule
