`default_nettype none

/* Relay pulsing */

module relay_pulse (clk,
					trig,
					sigout);
	
	parameter POL = 1'b1;
	parameter CLKFREQ = 24000000;
	parameter PULSELEN = 0.015;
	localparam integer NBCYCLE = CLKFREQ * PULSELEN;
	
	input wire clk;
	input wire trig;
	output wire sigout;
	
	reg cmdout = 1'b0;
	assign sigout = ~POL ^ cmdout;
	
	reg [1:0]trigedge = 2'b00;
	
	localparam IDLE			= 2'd0;
	localparam ACTIVE		= 2'd1;
	
	reg [1:0]state = IDLE;
	reg [31:0]counter = 32'd0;
	
	always @(posedge clk)
	begin
		cmdout <= 0;
		trigedge <= { trigedge[0], trig };
		case(state)
			IDLE: begin
				if(trigedge == 2'b01) state <= ACTIVE;
			end
			ACTIVE: begin
				cmdout <= 1'b1;
				counter <= counter + 32'd1;
				if(counter == NBCYCLE - 1) begin
					counter <= 32'd0;
					state <= IDLE;
				end
			end
		endcase
	end
	
endmodule