module spi_master(SCK, MOSI, MISO, CS,
					clk, dout, din, dvalid, dready, dsend);
	
	parameter WSIZE = 32;
	parameter CPOL = 0;
	parameter CPHA = 0;
	parameter OFLOW = 5;
	
	output reg SCK;
	output wire MOSI;
	input wire MISO;
	output reg CS;
	
	input wire clk;
	input wire [WSIZE-1:0]dout;
	output wire [WSIZE-1:0]din;
	output reg dvalid;
	output reg dready;
	input wire dsend;
	
	// Synchronise MISO
	(* ASYNC_REG = "TRUE" *)reg miso_0;
	(* ASYNC_REG = "TRUE" *)reg c_miso;
	always @(posedge clk)
	begin
		miso_0 <= MISO;
		c_miso <= miso_0;
	end
	
	// Generate the clock
	reg [7:0]counter = 0;
	reg raw_spi_clk = 0;
	
	always @(posedge clk)
	begin
		counter <= counter + 1;
		if(counter == OFLOW) begin
			counter <= 0;
			raw_spi_clk <= ~raw_spi_clk;
		end
	end
	
	wire spi_clk;
	
	if(CPOL == 1) begin // Clock idle at 1 we reverse it
		assign spi_clk = ~(raw_spi_clk);
	end else begin
		assign spi_clk = raw_spi_clk;
	end
	
	reg [WSIZE-1:0]dstore = 0;
	reg [WSIZE-1:0]drecv = 0;
	
	assign MOSI = dstore[WSIZE-1];
	assign din = drecv;
	
	localparam IDLE			= 0;
	localparam CSLOW			= 1;
	localparam PUSHDATA_H	= 2;
	localparam PUSHDATA_L	= 3;
	localparam CSHIGH			= 4;
	
	reg [3:0]state = 0;
	
	reg [31:0]bit_count = 0;
	
	reg [1:0]sck_edge = 2'b00;
	
	always @(posedge clk)
	begin
		SCK <= raw_spi_clk & ~CS;
		dready <= 0;
		CS <= 1;
		
		sck_edge <= {sck_edge[0], raw_spi_clk};
		
		case(state)
			IDLE: begin
				bit_count <= 0;
				dready <= 1;
				if(dsend) begin
					dready <= 0;
					state <= CSLOW;
					dstore <= dout;
				end
			end
			
			CSLOW: begin
				if(sck_edge == 2'b10) begin // Low we strobe CS
					CS <= 0;
					state <= PUSHDATA_H;
					// Data is already present in the two mode but it's ok
				end
			end
			
			PUSHDATA_H: begin
				CS <= 0;
				if(sck_edge == 2'b01) begin
					drecv <= {drecv[WSIZE-2:0], c_miso};
					state <= PUSHDATA_L;
				end
			end
			
			PUSHDATA_L: begin
				CS <= 0;
				if(sck_edge == 2'b10) begin
					dstore <= {dstore[WSIZE-2:0], 1'b0};
					bit_count <= bit_count + 1;
					state <= PUSHDATA_H;
					if(bit_count == WSIZE-1) begin
						state <= CSHIGH;
					end
				end
			end
			
			CSHIGH: begin
				if(sck_edge == 2'b01) begin
					CS <= 1;
					state <= IDLE;
					dready <= 1;
				end
			end
			
		endcase
	end
	
endmodule