`timescale 1ns / 1ps
`default_nettype none

module spi_slave(SCK, MISO, MOSI, CS
				clk, dout, din);
				
	parameter WORDSIZE = 32;
	parameter MSBFIRST = 1;
	
	input wire SCK;
	inout wire MISO;
	input wire MOSI;
	input wire CS;
	
	input wire clk;
	output reg [WORDSIZE-1:0]din;
	input wire [WORDSIZE-1:0]dout;
	
	reg rMISO = 0;
	assign MISO = !CS ? rMISO : 1'bZ;
	
	(* ASYNC_REG = "TRUE" *)reg sck_1 = 0;
	(* ASYNC_REG = "TRUE" *)reg sck_0 = 0;
	
	(* ASYNC_REG = "TRUE" *)reg mosi_1 = 0;
	(* ASYNC_REG = "TRUE" *)reg mosi_0 = 0;
	
	(* ASYNC_REG = "TRUE" *)reg cs_1 = 0;
	(* ASYNC_REG = "TRUE" *)reg cs_0 = 0;
	
	// Synchroniser
	always @(posedge clk)
	begin
		sck_1 <= SCK;
		sck_0 <= sck_1;
		
		mosi_1 <= MOSI;
		mosi_0 <= mosi_1;
		
		cs_1 <= CS;
		cs_0 <= cs_1;
	end
	
	// Transient suppressor
	wire c_sck = sck_0;
	wire c_mosi = mosi_0;
	wire c_cs = cs_0;
	
	reg [2:0]s_sck = 3'd0;
	reg [2:0]s_mosi = 3'd0;
	reg [2:0]s_cs = 3'd0;
	
	always @(posedge clk)
	begin
		s_sck <= {s_sck[1:0], c_sck};
		s_mosi <= {s_mosi[1:0], c_mosi};
		s_cs <= {s_cs[1:0], c_cs};
	end
	
	wire p_sck = (s_sck == 3'b111);
	
	wire p_mosi = (s_sck == 3'b111);
	
	wire p_cs = (s_sck == 3'b111);
	
	
	
endmodule