`timescale 1ns / 1ps
`default_nettype none

module top(
    input wire clk24,
	 // PLL SPI
    output wire PLL_CLK,
    output wire PLL_DATA,
    output wire PLL_LE,
    input wire PLL_LOCK,
	 // ADC SPI
	 output wire ADC_SCK,
	 output wire ADC_MOSI,
	 output wire ADC_CS,
	 input wire ADC_MISO,
    //output wire [7:0] EXT_BUS,
	 output wire [2:0]EXT_BUS,
	 output wire RELLOW,
	 output wire RELHIGH,
	 
	 input wire GEN_LOCK,
	 
	 output wire MIX1_NDIS,
	 output wire MIX1_X2,
	 output wire MIX1_EN,
	 
	 output wire MIX2_NDIS,
	 output wire MIX2_X2,
	 output wire MIX2_EN,
	 
	input wire FT_CLK,
	input wire NRXF,
	input wire NTXE,

	output wire NRD,
	output wire NWR,
	output wire NOE,
	output wire SIWU,
	
	inout wire [7:0]FDATA,
	
	// ADC BUS
	input wire DCO_P,
	input wire DCO_N,
	
	input wire FR_P,
	input wire FR_N,
	
	input wire AD_1A_P,
	input wire AD_1A_N,
	input wire AD_1B_P,
	input wire AD_1B_N,
	input wire AD_1C_P,
	input wire AD_1C_N,
	input wire AD_1D_P,
	input wire AD_1D_N,
	
	input wire AD_2A_P,
	input wire AD_2A_N,
	input wire AD_2B_P,
	input wire AD_2B_N,
	input wire AD_2C_P,
	input wire AD_2C_N,
	input wire AD_2D_P,
	input wire AD_2D_N,
	
	output wire ENC_P,
	output wire ENC_N
    );
	
	// Let's make a 8bit reg to drive all signals
	// 0: CHAN1 Enable (Mixer and opamp)
	// 1: CHAN2 Enable
	// 2: CHAN1 x2 Enable (For frequency above 6.8GHz)
	// 3: CHAN2 x2 Enable (For frequency above 6.8GHz)
	// 4: Freq Relay Position Low (Goes on external pin for now)
	// 5: Freq Relay Position High (Goes on external pin for now)
	// 6:
	// 7:
	
	wire [7:0]sig_driver;
	
	assign MIX1_NDIS = sig_driver[0];
	assign MIX1_X2 = sig_driver[2];
	assign MIX1_EN = sig_driver[0];
	
	assign MIX2_NDIS = sig_driver[1];
	assign MIX2_X2 = sig_driver[3];
	assign MIX2_EN = sig_driver[1];
	
	// Let's make a 8bit status register
	// 0: PLLREF LOCKED
	// 1: PLLGEN LOCKED
	
	(* ASYNC_REG = "true" *)reg [7:0]status = 0;
	(* ASYNC_REG = "true" *)reg [7:0]status_0 = 0;
	
	always @(posedge clk24)
	begin
		status_0 <= {6'b0, GEN_LOCK, PLL_LOCK};
		status <= status_0;
	end
	
	wire pulsehigh;
	wire pulselow;
	
	// Relay control
	relay_pulse #(.POL(1'b1), .CLKFREQ(32'd24000000), .PULSELEN(0.015)) highfreq (.clk(clk24), .trig(sig_driver[5]), .sigout(RELHIGH));
	relay_pulse #(.POL(1'b1), .CLKFREQ(32'd24000000), .PULSELEN(0.015)) highlow (.clk(clk24), .trig(sig_driver[4]), .sigout(RELLOW));
	
	// TO/FROM FTDI
	wire [7:0]FTDI_DIN;
	wire DIN_VALID;
	wire NINFIFO_READY;
	
	wire [7:0]FTDI_DOUT;
	wire NDOUT_VALID;
	wire FTDI_READY;
	
	// TO/FROM FSM
	wire [7:0]FSM_DIN;
	wire NINFIFO_VALID;
	wire INFSM_READY;
	
	wire [7:0]FSM_DOUT;
	wire FSM_DOUT_VALID;
	wire NOUTFIFO_READY;
	
	ccfifo ccinfifo (
	  .wr_clk(FT_CLK), // input wr_clk
	  .rd_clk(clk24), // input rd_clk
	  .din(FTDI_DIN), // input [7 : 0] din
	  .wr_en(DIN_VALID), // input wr_en
	  .rd_en(INFSM_READY), // input rd_en
	  .dout(FSM_DIN), // output [7 : 0] dout
	  .full(NINFIFO_READY), // output full
	  .wr_ack(), // output wr_ack
	  .empty(NINFIFO_VALID), // output empty
	  .valid() // output valid
	);
	
	ccfifo2 ccoutfifo (
	  .wr_clk(clk24), // input wr_clk
	  .rd_clk(FT_CLK), // input rd_clk
	  .din(FSM_DOUT), // input [7 : 0] din
	  .wr_en(FSM_DOUT_VALID), // input wr_en
	  .rd_en(FTDI_READY), // input rd_en
	  .dout(FTDI_DOUT), // output [7 : 0] dout
	  .full(NOUTFIFO_READY), // output full
	  .wr_ack(), // output wr_ack
	  .empty(NDOUT_VALID), // output empty
	  .valid() // output valid
	);
	
	ft2232_ft245 fifo_interface(.FT_CLK(FT_CLK), .NRXF(NRXF), .NTXE(NTXE), .NRD(NRD), .NWR(NWR), .NOE(NOE), .SIWU(SIWU), .FDATA(FDATA),
								.DIN(FTDI_DIN), .IVALID(DIN_VALID), .IREADY(!NINFIFO_READY),
								.DOUT(FTDI_DOUT), .OVALID(!NDOUT_VALID), .OREADY(FTDI_READY));
	
	// ADF4355
	wire [31:0]adf4355_word;
	wire adf4355_send;
	wire adf4355_ready;
	
	wire adf5355_send;
	wire adf5355_ready;
	
	// LT2271 SPI
	wire [15:0]lt2271_word;
	wire lt2271_send;
	wire lt2271_ready;
	
	// ADC Data
	wire read_adc_en;
	wire adc_data_valid;
	wire [12:0]nbsample_wanted;
	wire adc_capture_start;
	wire adc_busy;
	wire [7:0]adc_data;
	
	main_fsm ncpu(.clk(clk24),
				  .DIN(FSM_DIN), .IVALID(!NINFIFO_VALID), .IREADY(INFSM_READY),
				  .DOUT(FSM_DOUT), .OVALID(FSM_DOUT_VALID), .OREADY(!NOUTFIFO_READY),
				  .ADF4355_DOUT(adf4355_word), .ADF4355_SEND(adf4355_send), .ADF4355_READY(adf4355_ready), .ADF5355_SEND(adf5355_send), .ADF5355_READY(adf5355_ready),
				  .LT2271_DOUT(lt2271_word), .LT2271_SEND(lt2271_send), .LT2271_READY(lt2271_ready),
				  .ADC_BUSY(adc_busy), .ADC_DVALID(adc_data_valid), .ADC_READ(read_adc_en), .ADC_NSAMP(nbsample_wanted), .ADC_START(adc_capture_start), .ADC_DATA(adc_data),
				  .SIG_DRIVER(sig_driver), .STATUS(status));
				  
	spi_master ADF4355_SPI(.SCK(PLL_CLK), .MOSI(PLL_DATA), .MISO(), .CS(PLL_LE),
				  .clk(clk24), .dout(adf4355_word), .din(), .dready(adf4355_ready), .dsend(adf4355_send));
  
  wire GEN_SCK;
  wire GEN_MOSI;
  wire GEN_CS;
	
	assign EXT_BUS[2] = GEN_SCK;
	assign EXT_BUS[1] = GEN_MOSI;
	assign EXT_BUS[0] = GEN_CS;
  
	spi_master ADF5355_SPI(.SCK(GEN_SCK), .MOSI(GEN_MOSI), .MISO(), .CS(GEN_CS),
					.clk(clk24), .dout(adf4355_word), .din(), .dready(adf5355_ready), .dsend(adf5355_send));
	
	spi_master #(.WSIZE(16)) LT2271_SPI(.SCK(ADC_SCK), .MOSI(ADC_MOSI), .MISO(), .CS(ADC_CS),
					.clk(clk24), .dout(lt2271_word), .din(), .dready(lt2271_ready), .dsend(lt2271_send));
	
	/* LT2271 data interface */
	
	wire adc_clk;
	wire chan1_sync;
	wire [7:0]chan1_data;
	
	adc_interface adc_connect(.clklogic(clk24), .DCO_P(DCO_P), .DCO_N(DCO_N), .FR_P(FR_P), .FR_N(FR_N),
							.AD_1A_P(AD_1A_P), .AD_1A_N(AD_1A_N), .AD_1B_P(AD_1B_P), .AD_1B_N(AD_1B_N),
							.AD_1C_P(AD_1C_P), .AD_1C_N(AD_1C_N), .AD_1D_P(AD_1D_P), .AD_1D_N(AD_1D_N),
							.AD_2A_P(AD_2A_P), .AD_2A_N(AD_2A_N), .AD_2B_P(AD_2B_P), .AD_2B_N(AD_2B_N),
							.AD_2C_P(AD_2C_P), .AD_2C_N(AD_2C_N), .AD_2D_P(AD_2D_P), .AD_2D_N(AD_2D_N),
							.ENC_P(ENC_P), .ENC_N(ENC_N),
							.dout(adc_data), .dout_valid(adc_data_valid), .dread(read_adc_en), .dbusy(adc_busy), .dstart(adc_capture_start));

endmodule
